@@ -23,13 +23,14 @@ static gboolean l_connecting = FALSE;
 static Gda_Connection *l_database = NULL;
 static GtkWidget *w_dbname, *w_login, *w_pwd, *w_host,
                  *w_port, *w_options, *w_combo;
-static GtkWidget *w_dsnlist;
+static GtkWidget *w_dsnlist, *w_availlist;
 
 /* static function */
 static void 
 dialog_clicked_cb(GnomeDialog *dialog, gint button_number, 
                   gpointer data)
 {
+  Gda_Connection *db;
   register gint cnt;
   switch (button_number)
     {
@@ -40,7 +41,6 @@ dialog_clicked_cb(GnomeDialog *dialog, gint button_number,
 	  {
 	  case 0 : /* 'new' tab */
 	    {
-	      Gda_Connection *db;
 	      gchar *cur_driver, *drv_str, *dbname, *login, *pwd, 
 	            *host, *port, *options;
 	      /* get fields' values */
@@ -75,12 +75,56 @@ dialog_clicked_cb(GnomeDialog *dialog, gint button_number,
                     }
 		}
 	    }
-           case 1 : /* recent connections tab */
-             {
-             }
-             break;
-	   }
-         break;
+          case 1 : /* recent connections tab */
+          case 2 : /* available connections tab */
+            {
+              gchar user[64], password[64], provider[64], dsn[64];
+              gint sel;
+              if (button_number == 1)
+                sel = ui_get_clist_selection(GTK_CLIST(w_dsnlist));
+              else
+                sel = ui_get_clist_selection(GTK_CLIST(w_availlist));
+              if (sel >= 0)
+                {
+                  gint rc;
+                  if (ui_ask_user_password(user, password))
+                    {
+                      /* get GtkCList specific data */
+                      if (button_number == 1)
+                        {
+                          gtk_clist_get_text(GTK_CLIST(w_dsnlist), sel, 0, &provider);
+                          gtk_clist_get_text(GTK_CLIST(w_dsnlist), sel, 1, &dsn);
+                        }
+                      else
+                        {
+                          gtk_clist_get_text(GTK_CLIST(w_availlist), sel, 0, &provider);
+                          gtk_clist_get_text(GTK_CLIST(w_availlist), sel, 1, &dsn);
+                        }
+		      /* create driver connection */
+		      db = gda_connection_new(glb_orb);
+		      db->provider = g_strdup(provider);
+		      rc = gda_connection_open(db, dsn, user, password);
+		      if (rc == 0)
+		        {
+		          /* close dialog */
+		          l_connecting = FALSE;
+		          db_new_connection(db);
+		          app_add_recent_connection(provider, dsn, user);
+		          ui_status_bar_message("Connected to database %s",
+					        gda_connection_get_dsn(db));
+		          gnome_dialog_close(dialog);
+		        }
+		      else 
+                        {
+                          ui_show_error("Could not connect to database %s", dsn);
+                          gda_connection_free(db);
+                        }
+                    }
+                }
+              break;
+            }
+	  }
+        break;
       }
     case 1 : /* Cancel button */
       /* close dialog */
@@ -147,7 +191,7 @@ create_new_recent_tab (GtkWidget *notebook)
   GList *node;
   gchar *titles[] =
   {
-    _("Provider"), _("Database"), _("Login")
+    _("Provider"), _("DSN"), _("Login")
   };
   /* check parameters */
   g_return_val_if_fail(notebook != NULL, NULL);
@@ -182,12 +226,58 @@ create_new_recent_tab (GtkWidget *notebook)
   return (tab);
 }
 
+static GtkWidget *
+create_new_available_tab (GtkWidget *notebook)
+{
+  GtkWidget *tab;
+  GList *providers;
+  const gchar *titles[] = { _("Provider"), _("DSN") };
+  /* check parameters */
+  g_return_val_if_fail(notebook != NULL, NULL);
+  /* create the table */
+  tab = gtk_table_new(2, 4, FALSE);
+  gtk_widget_show(tab);
+  /* create list widget */
+  w_availlist = ui_new_clist_in_table(tab, 2, titles, 0, 0, 2, 4);
+  providers = gda_list_datasources();
+  if (providers != NULL)
+    {
+      GList *node;
+      while ((node = g_list_first(providers)) != NULL)
+        {
+          GList *dsns = gda_list_datasources_for_provider((gchar *) node->data);
+          if (dsns != NULL)
+            {
+              GList *dsn_node;
+              while ((dsn_node = g_list_first(dsns)) != NULL)
+                {
+                  GPtrArray *row = g_ptr_array_new();
+                  g_ptr_array_add(row, (gconstpointer) node->data);
+                  g_ptr_array_add(row, (gconstpointer) dsn_node->data);
+                  gtk_clist_set_column_auto_resize(GTK_CLIST(w_availlist), 0, TRUE);
+                  gtk_clist_set_column_auto_resize(GTK_CLIST(w_availlist), 1, TRUE);
+                  gtk_clist_set_column_auto_resize(GTK_CLIST(w_availlist), 2, TRUE);
+                  gtk_clist_append(GTK_CLIST(w_availlist), (gpointer) row->pdata);
+                  /* delete list node */
+                  dsns = g_list_remove(dsns, dsn_node->data);
+                  g_free(dsn_node->data);
+                  g_ptr_array_free(row, TRUE);
+                }
+            }
+          /* delete list node */
+          providers = g_list_remove(providers, node->data);
+          g_free(node->data);
+        }
+    }
+  return (tab);
+}
+
 /* function definitions */
 void 
 db_open_connection (GtkWidget *w, gpointer data)
 {
   register int cnt;
-  GtkWidget *notebook, *driver_tab, *recent_tab, *label;
+  GtkWidget *notebook, *driver_tab, *recent_tab, *avail_tab, *label;
   static GtkWidget *dialog;
   /* if we are already connecting... */
   if (l_connecting)
@@ -219,7 +309,7 @@ db_open_connection (GtkWidget *w, gpointer data)
       gtk_notebook_append_page(GTK_NOTEBOOK(notebook), driver_tab,
 			       label);
     }
-  /* create available connections tab */
+  /* create recent connections tab */
   recent_tab = create_new_recent_tab(notebook);
   if (recent_tab != NULL)
     {
@@ -227,6 +317,14 @@ db_open_connection (GtkWidget *w, gpointer data)
       label = gtk_label_new("Recent");
       gtk_notebook_append_page(GTK_NOTEBOOK(notebook), recent_tab, label);
     }
+  /* create available connections tab */
+  avail_tab = create_new_available_tab(notebook);
+  if (avail_tab != NULL)
+    {
+      /* create the tab's field */
+      label = gtk_label_new("Available");
+      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), avail_tab, label);
+    }
   gtk_widget_show(notebook);
   /* finally show the dialog */
   gtk_widget_show(dialog);
