@@ -61,17 +61,19 @@ grid_show_columns_cb (GtkWidget *w, gpointer data)
 #endif
 }
 
+#if defined(HAVE_GTKSHEET)
 void
 grid_hide_row_numbers_cb (GtkWidget *w, gpointer data)
 {
-  return;
+  gtk_sheet_hide_row_titles(GTK_SHEET(data));
 }
 
 void
 grid_show_row_numbers_cb (GtkWidget *w, gpointer data)
 {
-  return;
+  gtk_sheet_show_row_titles(GTK_SHEET(data));
 }
+#endif
 
 GnomeUIInfo gridpopupmenu[] =
 {
@@ -109,7 +111,7 @@ create_grid (gint cnt, gchar *titles[], GtkContainer *parent)
   register gint i;
   /* create widget */
 #if defined(HAVE_GTKSHEET)
-  grid = gtk_sheet_new(1, cnt, "A Title");
+  grid = gtk_sheet_new_browser(1, cnt, "A Title");
   for (i = 0; i < cnt; i++)
     {
       gtk_sheet_set_column_title(GTK_SHEET(grid), i, titles[i]);
@@ -126,6 +128,7 @@ create_grid (gint cnt, gchar *titles[], GtkContainer *parent)
   gtk_clist_set_selection_mode(GTK_CLIST(grid), GTK_SELECTION_SINGLE);
   gtk_clist_moveto(GTK_CLIST(grid), 0, 0, 0.0, 0.0);
   /* set grid for sorting */
+  gtk_clist_set_reorderable(GTK_CLIST(grid), TRUE);
   gtk_signal_connect(GTK_OBJECT(grid), "click_column",
 		     GTK_SIGNAL_FUNC(select_column_cb), NULL);
   gtk_clist_set_compare_func(GTK_CLIST(grid), 
@@ -220,8 +223,15 @@ grid_show_resultset (GtkWidget *old_grid, Gda_Recordset *rowset,
 	      if (field != NULL)
 		{
 #if defined(HAVE_GTKSHEET)
+                  gint pixels;
+                  GtkStyle *style;
+                  GdkFont *font;
+                  style = gtk_widget_get_style(grid);
+                  font = style->font;
                   gtk_sheet_set_cell(GTK_SHEET(grid), total_rows, col, 0, 
                                      gda_stringify_value(0, 0, field));
+                  pixels = gdk_string_width(font, gda_stringify_value(0, 0, field)) + 20;
+                  gtk_sheet_set_column_width(GTK_SHEET(grid), col, pixels);
 #else
 		  g_ptr_array_add(row, gda_stringify_value(0, 0, field));
 		  /* set column justification depending on field type */
