@@ -107,6 +107,12 @@ extern void browser_close_all (Gda_Connection *);
 struct GSQLFE_BrowserInfo *browser_get_current_window (void);
 extern void browser_new_window (GtkWidget *, gpointer );
 
+/*
+ * Database designer
+ */
+extern void designer_close_window (GtkWidget *, gpointer );
+extern void designer_open_window (GtkWidget *, gpointer );
+
 /*
  * SQL builder
  */
@@ -143,8 +149,10 @@ extern GtkWidget *grid_show_resultset (GtkWidget *, Gda_Recordset *, GtkContaine
 /* 
  * UI helper functions
  */
+extern gboolean ui_ask_user_password (gchar *, gchar *);
 extern void ui_fill_list (GtkList *, GList *);
 extern void ui_flash_message (const gchar *, ...);
+extern gint ui_get_clist_selection (GtkCList *);
 extern gchar *ui_get_file_name (gchar *);
 extern GtkWidget *ui_get_work_area (void);
 extern void ui_load_file_in_text (GtkText *, gchar *);
@@ -160,10 +168,12 @@ extern GtkWidget *ui_new_text_in_table (GtkWidget *, gint , gint ,
                                         gint , gint );
 extern GtkWidget *ui_new_toolbar_in_table (GtkWidget *, gint , gint , 
                                            gint , gint );
+extern GtkWidget *ui_new_tree_in_table (GtkWidget *, gint , gint , gint , gint );
 extern void ui_set_notebook_page (GtkNotebook *, GtkWidget *);
 extern void ui_show_error (const gchar *, ...);
 extern void ui_show_file_in_window (const gchar *);
 extern void ui_status_bar_message (const gchar *, ...);
+extern GtkWidget *ui_tree_new_subtree (GtkTree *, gchar *);
 
 /*
  * integrated text editor
