@@ -105,6 +105,7 @@ quit_cb (GtkWidget *w, gpointer data)
   editor_close_window(w, data);
   log_close_window(w, data);
   error_close_viewer(w, data);
+  designer_close_window(w, data);
   db_close_all_connections(w, data);
   app_save_settings(w, data);
   gtk_main_quit();
@@ -181,6 +182,9 @@ GnomeUIInfo databasemenu[] =
   { GNOME_APP_UI_ITEM, N_("SQL Builder"), N_("Construct SQL statements"),
     sql_new_builder, NULL, NULL, GNOME_APP_PIXMAP_NONE,
     NULL, 0, 0, NULL },
+  { GNOME_APP_UI_ITEM, N_("Database Designer"), N_("Visual Database Designer"),
+    designer_open_window, NULL, NULL, GNOME_APP_PIXMAP_NONE,
+    NULL, 0, 0, NULL },
   { GNOME_APP_UI_ITEM, N_("Error Viewer"), N_("View all errors in the connection"),
     error_open_viewer, NULL, NULL, GNOME_APP_PIXMAP_NONE,
     NULL, 0, 0, NULL },
