@@ -22,6 +22,53 @@
 extern GtkWidget *wWorkArea, *wStatusBar;
 gchar *glb_list_item_data_key = "list_item_data_key";
 
+/* ask user name and password */
+static void
+login_button_clicked (GtkWidget *w, glong *selected)
+{
+  *selected = (glong) gtk_object_get_data(GTK_OBJECT(w), "button");
+}
+
+gboolean
+ui_ask_user_password (gchar *user, gchar *password)
+{
+  GtkWidget *dialog;
+  glong *selected;
+  gboolean res;
+  /* initialize variables */
+  selected = (glong *) g_malloc(sizeof(glong));
+  *selected = 0;
+  /* create dialog */
+  dialog = gnome_dialog_new(N_("User Authentication"), GNOME_STOCK_BUTTON_OK,
+                            GNOME_STOCK_BUTTON_CANCEL, NULL);
+  gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);
+  gtk_object_set_data(GTK_OBJECT(g_list_nth(GNOME_DIALOG(dialog)->buttons, 0)),
+		      "button", (gpointer) 1);
+  gnome_dialog_button_connect(GNOME_DIALOG(dialog),
+		              "clicked", 
+		              GTK_SIGNAL_FUNC(login_button_clicked),
+		              (gpointer) selected);
+  gtk_object_set_data(GTK_OBJECT(g_list_nth(GNOME_DIALOG(dialog)->buttons, 1)),
+		      "button", (gpointer) 2);
+  gnome_dialog_button_connect(GNOME_DIALOG(dialog),
+		              "clicked", 
+		              GTK_SIGNAL_FUNC(login_button_clicked),
+		              (gpointer) selected);
+  gtk_widget_show(dialog);
+  /* wait for selection from user */
+  while (!*selected)
+    gtk_main_iteration_do(TRUE);
+  if (*selected == 1)
+    {
+      res = TRUE;
+    }
+  else res = FALSE;
+  /* destroy dialog */
+  gnome_dialog_close(GNOME_DIALOG(dialog));
+  g_free((gpointer) selected);
+  return (res);
+}
+
 void
 ui_fill_list (GtkList *wlist, GList *mlist)
 {
@@ -53,6 +100,15 @@ ui_flash_message (const gchar *format, ...)
   gnome_app_flash(GNOME_APP(glb_app), sz);
 }
 
+gint
+ui_get_clist_selection (GtkCList *clist)
+{
+  GList *sel;
+  g_return_val_if_fail(clist != NULL, -1);
+  sel = GTK_CLIST(clist)->selection;
+  return (sel != NULL ? GPOINTER_TO_INT(sel->data) : -1);
+}
+
 /* open file selection dialog and return selected file */
 static void
 filesel_button_clicked (GtkWidget *w, glong *selected)
@@ -402,6 +458,32 @@ ui_new_toolbar_in_table (GtkWidget *table, gint x1, gint y1, gint x2, gint y2)
   return (toolbar);
 }
 
+GtkWidget *
+ui_new_tree_in_table (GtkWidget *table, gint x1, gint y1, gint x2, gint y2)
+{
+  GtkWidget *scrolled_window, *tree;
+  g_return_val_if_fail(table != NULL, NULL);
+  /* create scrolled window to put tree inside */
+  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
+				 GTK_POLICY_AUTOMATIC,
+				 GTK_POLICY_AUTOMATIC);
+  gtk_table_attach(GTK_TABLE(table), 
+		   scrolled_window, x1, x2, y1, y2,
+		   GTK_FILL | GTK_SHRINK,
+		   GTK_SHRINK | GTK_EXPAND | GTK_FILL,
+		   3, 3);
+  gtk_widget_show(scrolled_window);
+  /* create the tree widget */
+  tree = gtk_tree_new();
+  gtk_tree_set_selection_mode(GTK_TREE(tree), GTK_SELECTION_SINGLE);
+  gtk_tree_set_view_lines(GTK_TREE(tree), TRUE);
+  gtk_widget_show(tree);
+  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), 
+					tree);
+  return (tree);
+}
+
 gboolean
 ui_save_text_to_file (GtkText *text, gchar *filename)
 {
@@ -485,3 +567,17 @@ ui_status_bar_message (const gchar *format, ...)
   va_end(args);
   gnome_appbar_set_status(GNOME_APPBAR(wStatusBar), (const gchar *) sz);
 }
+
+GtkWidget *
+ui_tree_new_subtree (GtkTree *tree, gchar *label)
+{
+  GtkWidget *item, *subtree;
+  g_return_val_if_fail(tree != NULL, NULL);
+  g_return_val_if_fail(label != NULL, NULL);
+  item = gtk_tree_item_new_with_label(label);
+  gtk_tree_append(GTK_TREE(tree), item);
+  subtree = gtk_tree_new();
+  gtk_tree_item_set_subtree(GTK_TREE_ITEM(item), subtree);
+  gtk_widget_show(item);
+  return (subtree);
+}
