@@ -22,6 +22,8 @@
 
 #include "gda-connection.h"
 #include "gda-server.h"
+#include "gda-error.h"
+#include "gda-command.h"
 
 static GHashTable* factories = 0;
 
@@ -78,7 +80,7 @@ get_corba_connection(Gda_Connection* cnc)
       fprintf(stderr,"Cannot open factory\n");
       return -1;
     }
-  cnc->connection = GDA_ConnectionFactory_create(factory, &ev);
+  cnc->connection = GDA_ConnectionFactory_create_connection(factory, "gda-odbc", &ev);
   if (Exception(&ev) < 0)
     return -1;
   return 0;
@@ -141,42 +143,6 @@ gda_connection_free(Gda_Connection* cnc)
   g_free(cnc);
 }  
 
-/**
- * gda_connection_list_provider:
- *
- * Searches the GNORBA database for GDA servers and returns a Glist of 
- * Gda_Server structs. 
- *
- * Returns: a GList of GDA servers structs
- */
-GList*
-gda_connection_list_providers(void)
-{
-  GList*          retval = 0;
-  GoadServerList* servlist;
-  GoadServer*     slist;
-  gint            i;
-  Gda_Server*     server;
-  
-  servlist = goad_server_list_get();
-  slist = servlist->list;
-  for (i = 0; slist[i].repo_id; i++)
-    {
-      if (strcmp(*slist[i].repo_id, "IDL:GDA/ConnectionFactory:1.0")  == 0)
-	{
-	  fprintf(stderr,"Found server '%s'\n", slist[i].server_id);
-	  server = gda_server_new();
-	  server->name     = g_strdup(slist[i].server_id);
-	  server->location = g_strdup(slist[i].location_info);
-	  server->comment  = g_strdup(slist[i].description);
-	  server->repo_id  = g_strdup(*(slist[i].repo_id));
-	  server->type     = slist[i].type;
-	  retval = g_list_append(retval, server);
-	}
-    }
-  goad_server_list_free(servlist);
-  return retval;
-}
 
 /**
  * gda_connection_set_provider:
@@ -237,6 +203,9 @@ gda_connection_list_datasources(Gda_Connection* cnc)
   if (!cnc->connection)
     get_corba_connection(cnc);
   dsnlist = GDA_Connection_listSources(cnc->connection, &ev);
+  if (!dsnlist || Exception(&ev))
+    return 0;
+  
   for (idx = 0; idx < dsnlist->_length; idx++)
     {
       gchar* dsn = g_strdup(dsnlist->_buffer[idx]);
@@ -298,14 +267,36 @@ gda_connection_open(Gda_Connection* cnc, gchar* dsn, gchar* user, gchar* pwd)
   g_return_val_if_fail(cnc->connection == CORBA_OBJECT_NIL, -1);
   g_return_val_if_fail(user != NULL, -1);
   g_return_val_if_fail(pwd != NULL, -1);
-
+  g_return_val_if_fail(cnc->is_open == 0, -1);
+  
   if (!dsn)
     db_to_use = cnc->default_db;
   else
-    db_to_use = dsn;
+    {
+      size_t pos = strcspn(dsn, ";");
+      if (pos != strlen(dsn))
+	db_to_use = g_strdup(dsn);
+      else
+	{
+	  gchar path[128];
+	  g_snprintf(path, sizeof(path), "/gdalib/%s/DSN", dsn);
+	  db_to_use = gnome_config_get_string(path);
+	}
+    }
   user_to_use = user;
   pwd_to_use = pwd;
 
+  if (!db_to_use)
+    {
+      Gda_Error* e = gda_error_new();
+
+      e->description = g_strdup(_("Database '%s' not found in system configuration"));
+      e->source = g_strdup("GDA Client Library");
+      e->native = g_strdup(e->description);
+      cnc->errors = g_list_append(cnc->errors, e);
+      return -1;
+    }
+      			 
   cnc->database = g_strdup(db_to_use);
   cnc->user = g_strdup(user_to_use);
   cnc->passwd = g_strdup(pwd_to_use);
@@ -368,16 +359,13 @@ open_schema(Gda_Connection* cnc, GDA_Connection_QType t,
   CORBA_Environment ev;
   CORBA_Object      corba_rs;
   Gda_Recordset*     rs;
-  gint               idx;
   
   CORBA_exception_init(&ev);
 
   corba_rs = GDA_Connection_openSchema(cnc->connection, t,constraints, &ev);
-  if (Exception(&ev) < 0)
+  if (Exception(&ev) < 0 || corba_rs == CORBA_OBJECT_NIL)
     return 0;
   rs = gda_recordset_new();
-  rs->eof = 0;
-  rs->bof = 0;
   rs->open = 1;
   rs->corba_rs = corba_rs;
   rs->field_attributes = GDA_Recordset_describe(corba_rs, &ev);
@@ -386,17 +374,21 @@ open_schema(Gda_Connection* cnc, GDA_Connection_QType t,
       fprintf(stderr," Cannot get recordset description\n");
     }
 #if 0
-  for (idx = 0; idx  < rs->field_attributes->_length; idx++)
-    {
-      gchar type_bfr[120];
-      fprintf(stderr,"Description for field '%s'\n", rs->field_attributes->_buffer[idx].name);
-      fprintf(stderr,"     defined Size   = %d\n", rs->field_attributes->_buffer[idx].definedSize);
-      fprintf(stderr,"     scale          = %d\n", rs->field_attributes->_buffer[idx].scale);
-      fprintf(stderr,"     gdaType        = '%s'\n", gda_gdatype_2_string(type_bfr, sizeof(type_bfr),
+  {
+    gint idx;
+    
+    for (idx = 0; idx  < rs->field_attributes->_length; idx++)
+      {
+	gchar type_bfr[120];
+	fprintf(stderr,"Description for field '%s'\n", rs->field_attributes->_buffer[idx].name);
+	fprintf(stderr,"     defined Size[prec] = %d\n", rs->field_attributes->_buffer[idx].definedSize);
+	fprintf(stderr,"     scale              = %d\n", rs->field_attributes->_buffer[idx].scale);
+	fprintf(stderr,"     gdaType            = '%s'\n", gda_gdatype_2_string(type_bfr, sizeof(type_bfr),
 									    rs->field_attributes->_buffer[idx].gdaType));
-      fprintf(stderr,"     cType          = %d\n", rs->field_attributes->_buffer[idx].cType);
-      fprintf(stderr,"     nativeType     = %d\n", rs->field_attributes->_buffer[idx].nativeType);
-    }
+	fprintf(stderr,"     cType              = %d\n", rs->field_attributes->_buffer[idx].cType);
+	fprintf(stderr,"     nativeType         = %d\n", rs->field_attributes->_buffer[idx].nativeType);
+      }
+  }
 #endif
   return rs;
 }
@@ -487,6 +479,7 @@ gda_connection_open_schema(Gda_Connection* cnc, GDA_Connection_QType t, ...)
   GList*                        ptr;
   gint                          count = 0;
   gint                          index;
+  Gda_Recordset*                rc;
   
   g_return_val_if_fail(cnc != 0, 0);
   g_return_val_if_fail(cnc->connection != 0, 0);
@@ -500,9 +493,10 @@ gda_connection_open_schema(Gda_Connection* cnc, GDA_Connection_QType t, ...)
       if (constraint_type == GDA_Connection_no_CONSTRAINT)
 	break;
       constraint_value = va_arg(ap, char*);
+      fprintf(stderr,"gda_connection_open_schema: constraint value = '%s'\n", constraint_value);
       c = g_new0(GDA_Connection_Constraint, 1);
       c->ctype = constraint_type;
-      c->value = g_strdup(constraint_value);
+      c->value = CORBA_string_dup(constraint_value);
       tmp_constraints = g_list_append(tmp_constraints, c);
       count++;
     }
@@ -517,15 +511,18 @@ gda_connection_open_schema(Gda_Connection* cnc, GDA_Connection_QType t, ...)
     {
       memcpy(&constraints->_buffer[index], ptr->data,
 	     sizeof(GDA_Connection_Constraint));
+      fprintf(stderr,"CORBA seq: constraint->value = '%s'\n", constraints->_buffer[index].value);
       index++;
       count--;
+      c = ptr->data;
+      g_free(c);
       ptr = g_list_next(ptr);
     }
+  g_list_free(tmp_constraints);
   CORBA_exception_init(&ev);
-  /* @mla@ FIXME: free allocated storage */
-  g_warning("gda_connection_open_schema: Leaking ememory");
-  return open_schema(cnc, t, constraints);
-
+  rc = open_schema(cnc, t, constraints);
+  CORBA_free(constraints);
+  return rc;
 }
 
 /**
@@ -548,14 +545,46 @@ gda_connection_open_schema(Gda_Connection* cnc, GDA_Connection_QType t, ...)
 GList*
 gda_connection_get_errors(Gda_Connection* cnc)
 {
-  GList* retval;
+  GList*             retval;
+  GDA_ErrorSeq*      remote_errors;
+  CORBA_Environment  ev;
+  gint               idx;
 
+  g_return_val_if_fail(cnc != NULL, 0);  
+  CORBA_exception_init(&ev);
+  
+  if (cnc->connection)
+    {
+      remote_errors = GDA_Connection__get_errors(cnc->connection, &ev);
+      Exception(&ev);
+      for (idx = 0; idx < remote_errors->_length; idx++)
+	{
+	  Gda_Error* e;
+	  e = gda_error_new();
+	  e->description = g_strdup(remote_errors->_buffer[idx].description);
+	  e->number      = remote_errors->_buffer[idx].number;
+	  e->source      = g_strdup(remote_errors->_buffer[idx].source);
+	  e->helpurl = 0;
+	  e->sqlstate    = g_strdup(remote_errors->_buffer[idx].sqlstate);
+	  e->native      = g_strdup(remote_errors->_buffer[idx].nativeMsg);
+	  cnc->errors = g_list_append(cnc->errors, e);
+	}
+      CORBA_free(remote_errors);
+    }
+  
   retval = cnc->errors;
   cnc->errors = 0;
   return retval;
 }
 
-
+/**
+ * gda_connection_begin_transaction:
+ * @cnc: the connection object
+ *
+ * Start a new transaction on @cnc
+ *
+ * Returns: 0 if everything is okay, -1 on error
+ */
 gint
 gda_connection_begin_transaction(Gda_Connection* cnc)
 {
@@ -570,6 +599,14 @@ gda_connection_begin_transaction(Gda_Connection* cnc)
 }
 
 
+/**
+ * gda_connection_commit_transaction:
+ * @cnc: the connection object
+ *
+ * Commit a pending transaction on @cnc
+ *
+ * Returns: 0 if everything is okay, -1 on error
+ */
 gint
 gda_connection_commit_transaction(Gda_Connection* cnc)
 {
@@ -584,6 +621,14 @@ gda_connection_commit_transaction(Gda_Connection* cnc)
   return 0;
 }
 
+/**
+ * gda_connection_abort_transaction:
+ * @cnc: the connection object
+ *
+ * Abort and rollback a pending transaction on @cnc
+ *
+ * Returns: 0 if everything is okay, -1 on error
+ */
 
 gint
 gda_connection_rollback_transaction(Gda_Connection* cnc)
@@ -598,3 +643,106 @@ gda_connection_rollback_transaction(Gda_Connection* cnc)
   return 0;
   return 0;
 }
+
+
+/**
+ * gda_connection_execute:
+ * @cnc: the conneciton
+ * @txt: the statement to execute
+ * @reccount: a pointer to a gulong. The number of affected records is 
+ * stored in the space where the pointer points to
+ * @flags: falgs (currently unused
+ *
+ * Executes @txt. An internal Gda_Command object is used for
+ * execution. This command object is destroyed automatically after the 
+ * commadn has finished.
+ *
+ * Returns: a recordset on success or NULL on error.
+ */
+Gda_Recordset*
+gda_connection_execute(Gda_Connection* cnc, gchar* txt, gulong* reccount, gulong flags)
+{
+  Gda_Command*   cmd;
+  Gda_Recordset* rs;
+  
+  cmd = gda_command_new();
+  gda_command_set_connection(cmd, cnc);
+  gda_command_set_text(cmd, txt);
+  rs = gda_command_execute(cmd, reccount, flags);
+  gda_command_free(cmd);
+  return rs;
+}
+
+/**
+ * gda_connection_start_logging:
+ * @cnc: the connection
+ * @filename: the name of the logfile
+ *
+ * This function causes the server to log the statements
+ * it executes in @filename.
+ *
+ * Returns: -1 on error and 0 if everything is okay
+ */
+gint
+gda_connection_start_logging(Gda_Connection* cnc, gchar* filename)
+{
+  CORBA_Environment ev;
+  gint              rc;
+  
+  g_return_val_if_fail(cnc, -1);
+  g_return_val_if_fail(filename, -1);
+  g_return_val_if_fail(cnc->connection, -1);
+
+  CORBA_exception_init(&ev);
+  rc = GDA_Connection_startLogging(cnc->connection, filename, &ev);
+  if (Exception(&ev) || rc < 0)
+    {
+      return -1;
+    }
+  return 0;
+}
+
+/**
+ * gda_connection_start_logging:
+ * @cnc: the connection
+ * @filename: the name of the logfile
+ *
+ * This function causes the server to log the statements
+ * it executes in @filename.
+ *
+ * Returns: -1 on error and 0 if everything is okay
+ */
+gint
+gda_connection_stop_logging(Gda_Connection* cnc)
+{
+  CORBA_Environment ev;
+  gint              rc;
+  
+  g_return_val_if_fail(cnc, -1);
+  g_return_val_if_fail(cnc->connection, -1);
+
+  CORBA_exception_init(&ev);
+  rc = GDA_Connection_stopLogging(cnc->connection, &ev);
+  if (Exception(&ev) || rc < 0)
+    {
+      return -1;
+    }
+  return 0;
+}
+  
+    
+gchar*
+gda_connection_create_recordset(Gda_Connection* cnc, Gda_Recordset* rs)
+{
+
+  gchar*            retval;
+  CORBA_Environment ev;
+
+  CORBA_exception_init(&ev);
+  retval = GDA_Connection_createTable(cnc->connection,
+				      rs->name,
+				      rs->field_attributes, &ev);
+  if (Exception(&ev))
+    return 0;
+  return retval;
+}
