@@ -123,6 +123,22 @@ gda_stringify_value(gchar* bfr, gint maxlen, Gda_Field* f)
 	  retval = g_strdup(retstr);
       }
       break;
+    case GDA_TypeVarbin:
+      {
+	gint len = maxlen - 1;
+	gint copylen;
+	
+	if (!bfr)
+	  {
+	    retval = g_new0(char, f->real_value->_u.v._u.lvb._length + 1);
+	    len = f->real_value->_u.v._u.lvb._length;
+	  }
+	copylen = min(len, f->real_value->_u.v._u.lvb._length);
+	fprintf(stderr,"GDA_TypeVarbin: Copying %d bytes\n", copylen);
+	memcpy(retval, f->real_value->_u.v._u.lvb._buffer, copylen);
+	retval[len] = '\0';
+      }
+      break;
     case GDA_TypeLongvarchar:
     case GDA_TypeVarchar:
       if (!bfr)
@@ -196,7 +212,6 @@ gda_stringify_value(gchar* bfr, gint maxlen, Gda_Field* f)
     case GDA_TypeTinyint:
     case GDA_TypeUBigint:
     case GDA_TypeUSmallint:
-    case GDA_TypeVarbin:
     case GDA_TypeVarwchar:
     case GDA_TypeFixchar:
     case GDA_TypeFixbin:
