@@ -32,7 +32,7 @@ typedef struct _Gda_Field
 
 
 
-#define gda_field_isnull(f)         ((f)->real_value->_d)
+#define gda_field_isnull(f)         (f->real_value ? (f)->real_value->_d : 1)
 #define gda_field_typecode(f)       ((f)->real_value->_u._d)
 #define gda_field_tinyint(f)        ((f)->real_value->_u.v._u.c)
 #define gda_field_typechar(f)       ((f)->real_value->_u.v._u.c1)
