@@ -65,6 +65,11 @@ gint                        gda_mysql_connection_open(Gda_MYSQL_Connection* cnc,
 						     
 gint                        gda_mysql_connection_close(Gda_MYSQL_Connection* cnc);
 
+Gda_MYSQL_Recordset*        gda_mysql_connection_open_schema(Gda_MYSQL_Connection*      cnc,
+							     Gda_MYSQL_Error*           e,
+							     GDA_Connection_QType       t,
+							     GDA_Connection_Constraint* constraints,
+							     gint                       length);
 #endif
 
 
