@@ -27,5 +27,7 @@
 
 
 Gda_MYSQL_Recordset*    gda_mysql_recset_new                  (void);
-
+void                    gda_mysql_recset_free                 (Gda_MYSQL_Recordset* recset);
+gint                    gda_mysql_recset_describe             (Gda_MYSQL_Recordset* recset);
+gint                    gda_mysql_recset_close                (Gda_MYSQL_Recordset* recset);
 #endif
