@@ -1,3 +1,21 @@
+/* GNOME DB
+ * Copyright (C) 1998 Michael Lausch
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free
+ * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
+
 #include <orb/orbit.h>
 #include "gda-mysql-types.h"
 #include "gda.h"
@@ -73,16 +91,18 @@ POA_GDA_Connection__epv poa_gda_connection__epv = {
   (gpointer)14,                         /* set_isOpen */
   (gpointer)15,				/* get_version */
   (gpointer)16,				/* get_errors */
-  (gpointer)17,				/* set_errors */
   (gpointer)18,				/* begin Transaction */
   (gpointer)19,				/* commit Transaction */
   (gpointer)20,				/* rollback Transaction */
-  gda_mysql_Connection_close,			/* close */
+  gda_mysql_Connection_close,		/* close */
   (gpointer)22,				/* execute */
-  gda_mysql_Connection_open,		        /* open */
-  gda_mysql_Connection_openSchema, 	        /* openSchema */
-  gda_mysql_Connection_createCommand,         /* createCommand */
-  gda_mysql_Connection_listSources            /* listSources */
+  gda_mysql_Connection_open,		/* open */
+  gda_mysql_Connection_openSchema, 	/* openSchema */
+  gda_mysql_Connection_createCommand,   /* createCommand */
+  gda_mysql_Connection_listSources,     /* listSources */
+  (gpointer)23,			        /* openRecordset */
+  (gpointer)24,                         /* startLogging */
+  (gpointer)25                          /* stoplogging */
 };
 
 
@@ -145,26 +165,27 @@ typedef struct
   Gda_MYSQL_Command*  cmd;
 }GDACommand_Servant;
 
-static GDA_Recordset_Chunk* gda_mysql_Recset_fetch(PortableServer_Servant recset, CORBA_long count,  CORBA_Environment* ev);
+static GDA_Recordset_Chunk* gda_mysql_Recset_fetch(PortableServer_Servant _obj, CORBA_long count,  CORBA_Environment* ev);
+static GDA_RowAttributes* gda_mysql_Recset_describe(PortableServer_Servant _obj, CORBA_Environment* ev);
 
 POA_GDA_Recordset__epv poa_gda_recordset__epv = {
-  NULL,				/* void *_private */
+  NULL,				        /* void *_private */
   (gpointer)1,				/* get_myConnection */
   (gpointer)2,				/* set_myConnection */
   (gpointer)3,				/* get_bof */
-  (gpointer)4,                            /* get_eof */
-  (gpointer)5,                            /* get_currentBookmark */
-  (gpointer)6,                            /* set_currentBookmark */
-  (gpointer)7,                            /* get_cachesize */
+  (gpointer)4,                          /* get_eof */
+  (gpointer)5,                          /* get_currentBookmark */
+  (gpointer)6,                          /* set_currentBookmark */
+  (gpointer)7,                          /* get_cachesize */
   (gpointer)8,				/* set_cachesize */
   (gpointer)9,				/* get_cursorLoc */
   (gpointer)10,				/* set_cursorLoc */
-  (gpointer)11,                           /* get_currentCursorType */
-  (gpointer)12,                           /* set_currentCursorType */
+  (gpointer)11,                         /* get_currentCursorType */
+  (gpointer)12,                         /* set_currentCursorType */
   (gpointer)13,				/* get_lockingMode */
   (gpointer)14,				/* set_lockingMode */
   (gpointer)15,				/* get_marshalling */
-  (gpointer)16,                           /* set_marshalling */
+  (gpointer)16,                         /* set_marshalling */
   (gpointer)17,				/* get_maxrecords */
   (gpointer)18,				/* set_maxrecords */
   (gpointer)19,				/* get_pagecount */
@@ -194,6 +215,7 @@ POA_GDA_Recordset__epv poa_gda_recordset__epv = {
   (gpointer)44,			        /* itemName */
   (gpointer)43,				/* dump */
   gda_mysql_Recset_fetch,	        /* fetch */
+  gda_mysql_Recset_describe,            /* describe */
 };
 
 POA_GDA_Recordset__vepv poa_gda_recordset__vepv =
