@@ -28,14 +28,14 @@
 #include "gda-error.h"
 #include "gda-command.h"
 #include "gda-field.h"
-
+#include "gda-odbc-misc.h"
 
 
 Gda_ODBC_Recordset*
 gda_odbc_recset_new()
 {
   Gda_ODBC_Recordset* rc = g_new0(Gda_ODBC_Recordset, 1);
-
+  
   return rc;
 }
 
@@ -50,112 +50,16 @@ gda_odbc_recset_free(Gda_ODBC_Recordset* recset)
 
 
 
-GDA_ValueType
-gda_odbc_sql2gdatype(gint sql_type)
-{
-  switch (sql_type)
-    {
-    case SQL_BIT:
-    case SQL_BIGINT:
-      return GDA_TypeBigint;
-    case SQL_SMALLINT:
-      return GDA_TypeSmallint;
-    case SQL_TINYINT:
-      return GDA_TypeTinyint;
-    case SQL_INTEGER:
-      return GDA_TypeInteger;
-    case SQL_CHAR:
-      return GDA_TypeChar;
-    case SQL_VARCHAR:
-      return GDA_TypeVarchar;
-    case SQL_VARBINARY:
-      return GDA_TypeVarbin;
-    case SQL_BINARY:
-      return GDA_TypeBinary;
-    case SQL_LONGVARBINARY:
-      return GDA_TypeLongvarbin;
-    case SQL_LONGVARCHAR:
-      return GDA_TypeLongvarchar;
-    case SQL_DECIMAL:
-      return GDA_TypeDecimal;
-    case SQL_NUMERIC:
-      return GDA_TypeNumeric;
-    case SQL_REAL:
-      return GDA_TypeSingle;
-    case SQL_DOUBLE:
-      return GDA_TypeDouble;
-    case SQL_FLOAT:
-      return GDA_TypeSingle;
-    case SQL_DATE:
-      return GDA_TypeDbDate;
-    case SQL_TIME:
-      return GDA_TypeDbTime;
-    case SQL_TIMESTAMP:
-      return GDA_TypeDbTimestamp;
-    default:
-      return GDA_TypeNull;
-    }
-}
 
 gint
-gda_odbc_sql2ctype(gint sql_type)
-{
-  switch (sql_type)
-    {
-    case SQL_BIT:
-      return SQL_C_BIT;
-    case SQL_BIGINT:
-      return SQL_C_CHAR;
-    case SQL_SMALLINT:
-      return SQL_C_SSHORT;
-    case SQL_TINYINT:
-      return SQL_C_STINYINT;
-    case SQL_INTEGER:
-      return SQL_C_SLONG;
-    case SQL_CHAR:
-      return SQL_C_CHAR;
-    case SQL_VARCHAR:
-      return SQL_C_CHAR;
-    case SQL_VARBINARY:
-      return SQL_C_BINARY;
-    case SQL_BINARY:
-      return SQL_C_BINARY;
-    case SQL_LONGVARBINARY:
-      return SQL_C_BINARY;
-    case SQL_LONGVARCHAR:
-      return SQL_C_CHAR;
-    case SQL_DECIMAL:
-      return SQL_C_CHAR;
-    case SQL_NUMERIC:
-      return SQL_C_CHAR;
-    case SQL_REAL:
-      return SQL_C_FLOAT;
-    case SQL_DOUBLE:
-      return SQL_C_DOUBLE;
-    case SQL_FLOAT:
-      return SQL_C_DOUBLE;
-    case SQL_DATE:
-      return SQL_C_DATE;
-    case SQL_TIME:
-      return SQL_C_TIME;
-    case SQL_TIMESTAMP:
-      return SQL_C_TIMESTAMP;
-    default:
-      return GDA_TypeNull;
-    }
-}
-
-static gint
-allocate_recordset_memory(Gda_ODBC_Recordset* rec)
+gda_odbc_recset_allocmem(Gda_ODBC_Recordset* rec)
 {
   Gda_ODBC_Field* server_field;
   GList*          ptr = rec->fields;
   gint            rc;
   gint            i;
-  Gda_ODBC_Error* e;
-  
+
   i = 0;
-  
   while (ptr)
     {
       server_field = ptr->data;
@@ -166,10 +70,8 @@ allocate_recordset_memory(Gda_ODBC_Recordset* rec)
 		    0);
 	  return -1;
 	}
-      server_field->value->_d = gda_odbc_sql2gdatype(server_field->sql_type);
-
+      server_field->value->_d = gda_odbc_sql2gdatype(server_field->sql_type);      
       server_field->c_type = gda_odbc_sql2ctype(server_field->sql_type);
-      
       switch (server_field->c_type)
 	{
 	  /* GDA_TypeBigint, GDA_TypeChar, GDA_TypeCurrency,
@@ -190,7 +92,6 @@ allocate_recordset_memory(Gda_ODBC_Recordset* rec)
 	       If that's enough, fine. Otherwise the client must use the get_chunk
 	       method to retrieve the data used for (BLOBS).
 	    */
-	    fprintf(stderr,"SQL_C_CHAR: server_field->sql_type = %d\n", server_field->sql_type);
 	    switch (server_field->sql_type)
 	      {
 	      case SQL_LONGVARBINARY:
@@ -308,7 +209,7 @@ allocate_recordset_memory(Gda_ODBC_Recordset* rec)
 			  server_field->value->_u.lvb._maximum,
 			  (glong*)&server_field->actual_length);
 	  if (rc != 0)
-	    goto bad_binding;
+	      goto bad_binding;
 	  break;
 	  
 	case SQL_C_DATE:	/* GDA_DbDate */
@@ -383,8 +284,21 @@ allocate_recordset_memory(Gda_ODBC_Recordset* rec)
     }
   return 0;
  bad_binding:
-  e = gda_odbc_error_new();
-  gda_odbc_error_make(e, rec, 0, "SQLBindCol");
+  {
+    gint idx;
+    ptr = rec->fields;
+    for (idx = 0; idx < i; idx++)
+      {
+	server_field = ptr->data;
+	if (server_field->c_type == SQL_C_CHAR)
+	  CORBA_free(server_field->value->_u.lvc);
+	if (server_field->c_type == SQL_C_BINARY)
+	  CORBA_free(server_field->value->_u.lvb._buffer);
+	server_field->value->_d = 1;
+	ptr = g_list_next(ptr);
+      }
+  }
+  gda_odbc_error_make( rec, 0, "SQLBindCol");
   return -1;
 }
 
@@ -395,13 +309,18 @@ gda_odbc_recset_move_next(Gda_ODBC_Recordset* recset)
 {
   gint rc;
 
-  recset->at_begin = 0;
-  allocate_recordset_memory(recset);
-
+  if (recset->at_end)
+    {
+      fprintf(stderr,"EOF already reached\n");
+      return 1;
+    }
+  if (gda_odbc_recset_allocmem(recset) < 0)
+    {
+      fprintf(stderr,"gda_odbc_recset_move_next: gda_odbc_recset_allocmem returns -1\n");
+      return -1;
+    }
   do {
-    
     rc = SQLFetch(recset->hstmt);
-    
     if (rc == SQL_NO_DATA_FOUND)
       {
 	recset->at_end = 1;
@@ -409,11 +328,13 @@ gda_odbc_recset_move_next(Gda_ODBC_Recordset* recset)
       }
     if (rc == SQL_ERROR)
       {
+	gda_odbc_error_make(recset, 0, "SQLFetch");
 	return -1;
       }
     if (rc == SQL_INVALID_HANDLE)
       {
-	;
+	gda_odbc_error_make(recset, 0, "SQLFetch");
+	return -1;
       }
   }while (recset->internal_filter_function && !recset->internal_filter_function(recset, 0));
   recset->pos++;
@@ -452,18 +373,12 @@ gda_odbc_recset_move_prev(Gda_ODBC_Recordset* recset)
     }
   if (rc == SQL_INVALID_HANDLE)
     {
-      ;
+      return -1;
     }
   recset->pos--;
   return 0;
 }
 
-void
-gda_odbc_recset_dump(Gda_ODBC_Recordset* rec)
-{
-  ;
-}
-
 gint
 gda_odbc_recset_close(Gda_ODBC_Recordset* rec)
 {
@@ -494,3 +409,5 @@ gda_odbc_recset_close(Gda_ODBC_Recordset* rec)
   fprintf(stderr,"gda_odbc_recset_close: left\n");
   return 0;
 }
+
+
