@@ -1144,5 +1144,21 @@ RETCODE SQLBindParameter(
         SDWORD  cbValueMax,
         SDWORD* pcbValue);
 
+RETCODE SQLSetConnectOption(
+	HDBC   hdbc,
+	UWORD  fOption,
+	UDWORD vParam);
+
+RETCODE SQLProcedures(
+	HSTMT    hstmt,
+	UCHAR*   szProcQualifier,
+	SWORD    cbProcQualifier,
+	UCHAR*   szProcOwner,
+	SWORD    cbProcOwner,
+	UCHAR*   szProcName,
+	SWORD    dbProcName);
+
+RETCODE SQLFetchPrev(
+	HSTMT    hstmt);
 
 #endif /* CLI0DEFS_H */
