@@ -31,27 +31,33 @@ fill_field_values (Gda_POSTGRES_Recordset *recset, glong row)
   g_return_val_if_fail(recset->cmd != NULL, FALSE);
   g_return_val_if_fail(recset->cmd->pq_data != NULL, FALSE);
   /* traverse all recordset values */
+
   node = g_list_first(recset->fields);
+
+  fprintf(stderr,"getting data from row %d\n", row);
   while (node != NULL)
     {
       Gda_POSTGRES_Field *field = (Gda_POSTGRES_Field *) node->data;
       if (field != NULL)
         {
           gchar *native_value;
-          if (field->value != NULL) 
-            {
-              if (field->value->_u.lvc != NULL) g_free(field->value->_u.lvc);
-              g_free(field->value);
-            }
-          field->value = g_new0(GDA_Value, 1);
           /* FIXME: must map Postgres value to GDA value */
           field->value->_d = GDA_TypeVarchar;
           /* get field's value from server */
-          native_value = PQgetvalue(recset->cmd->pq_data, row, cnt);
-          if (native_value != NULL)
-            field->value->_u.lvc = g_strdup(native_value);
-          field->value->_u.lvc = g_strdup("<NULL>");
-        }
+
+	  native_value = PQgetvalue(recset->cmd->pq_data, row, cnt);
+	  field->value->_d = GDA_TypeVarchar;
+	  if (native_value != NULL)
+	    {
+	      field->actual_length = strlen(native_value) + 1;
+	      field->value->_u.lvc = CORBA_string_dup(native_value);
+	    }
+	  else
+	    {
+	      field->actual_length = 0;
+	      field->value->_u.lvc = 0;
+	    }
+	}
       node = g_list_next(node);
       cnt++;
     }
@@ -79,10 +85,12 @@ gda_postgres_recset_free (Gda_POSTGRES_Recordset *recset)
 gint
 gda_postgres_recset_move_next (Gda_POSTGRES_Recordset *recset)
 {
+  gint ntuples = PQntuples(recset->cmd->pq_data);
+
   /* check if we can fetch forward */
   if (recset->pos < PQntuples(recset->cmd->pq_data))
     {
-      if (fill_field_values(recset, recset->pos + 1))
+      if (fill_field_values(recset, recset->pos))
         {
           recset->at_end = recset->at_begin = 0;
           recset->pos++;
@@ -141,4 +149,4 @@ gda_postgres_recset_close (Gda_POSTGRES_Recordset *recset)
 void
 gda_postgres_recset_dump (Gda_POSTGRES_Recordset *recset)
 {
-}
\ No newline at end of file
+}
