@@ -699,6 +699,37 @@ Exception_Function (CORBA_Environment* ev)
     }
   return 0;
 }
+static GDA_Error*
+make_errorbuffer(Gda_POSTGRES_Connection* cnc)
+{
+  gint          idx;
+  GList*        ptr;
+  GDA_Error*    rc;
+
+  rc  = CORBA_sequence_GDA_Error_allocbuf(g_list_length(cnc->errors));
+  idx = 0;
+  ptr = cnc->errors;
+  
+  while(ptr)
+    {
+      Gda_POSTGRES_Error* e = ptr->data;
+      rc[idx].description = CORBA_string_dup(e->description);
+      rc[idx].number = e->number;
+      rc[idx].source = CORBA_string_dup(e->source);
+#if 0
+      rc->_buffer[idx].helpfile = CORBA_string_dup(e->helpfile);
+      rc->_buffer[idx].helpctxt = CORBA_string_dup(e->helpctxt);
+#endif
+      rc[idx].sqlstate = CORBA_string_dup(e->sqlstate);
+      rc[idx].nativeMsg   = CORBA_string_dup(e->native);
+      gda_postgres_error_free(e);
+      ptr = g_list_next(ptr);
+      idx++;
+    }
+  g_list_free(cnc->errors);
+  cnc->errors = 0;
+  return rc;
+}      
 
 /*** Stub implementations ***/
 static GDA_Recordset 
@@ -1179,7 +1210,7 @@ impl_GDA_Connection_createRecordset(impl_POA_GDA_Connection * servant,
 				    CORBA_Environment * ev)
 {
    GDA_Recordset retval;
-
+   retval = 0;
    return retval;
 }
 
@@ -1611,108 +1642,97 @@ impl_GDA_Recordset_fetch(impl_POA_GDA_Recordset * servant,
   gint                    rowidx = 0;
   gint                    colidx;
   gint                    rc;
+  GList*                  tmp_rows;
+  gint                    rowlength;
   
-  chunk = GDA_Recordset_Chunk__alloc();
-  chunk->_buffer = CORBA_sequence_GDA_Row_allocbuf(count);
-  chunk->_length = 0;
-  for (rowidx = 0; rowidx < count; rowidx++)
-    {
-      chunk->_buffer[rowidx]._length = 0;
-      chunk->_buffer[rowidx]._buffer = 0;
-    }
+  if (!servant->recset)
+    return CORBA_OBJECT_NIL;
+  
+  tmp_rows = 0;
   rowidx = 0;
+  rowlength = g_list_length(rs->fields);
   
   do {
-      gint  rowlength = g_list_length(rs->fields);
-      ptr = rs->fields;
+    row = g_new0(GDA_Row, 1);
+    row->_buffer = CORBA_sequence_GDA_Field_allocbuf(rowlength);
+    row->_length = rowlength;
+    ptr = rs->fields;
+
+    colidx = 0;
+
+    while (ptr)
+      {
+	
+	server_field = ptr->data;
+
+	server_field->value = &row->_buffer[colidx].realValue._u.v;
+	row->_buffer[colidx].realValue._d = 1;
+	row->_buffer[colidx].shadowValue._d = 1;
+	row->_buffer[colidx].originalValue._d = 1;
+	ptr = g_list_next(ptr);
+	colidx++;
+      }
+    
+    rc = gda_postgres_recset_move_next(rs);
       
-      row = &chunk->_buffer[rowidx];
-      row->_buffer = CORBA_sequence_GDA_Field_allocbuf(rowlength);
-      row->_length = rowlength;
-      row->_maximum = 0;
-      field = row->_buffer;
+    if (rc != 0)
+      {
+	CORBA_free(row->_buffer);
+	g_free(row);
+	break;
+      }
+    tmp_rows = g_list_append(tmp_rows, row);
 
-      while(ptr)
-	{
-	  field->realValue._d = 1;
-	  server_field->name = 0;
-	  server_field = ptr->data;
-	  server_field->value = &field->realValue._u.v;
-	  server_field->value->_u.lvc = 0;
-	  ptr = g_list_next(ptr);
-	  field++;
-	}
+    ptr = rs->fields;
+    field = &row->_buffer[0];
+    colidx = 0;
 
-      rc = gda_postgres_recset_move_next(rs);
-      
-      if (rc != 0)
-	{
-	  chunk->_length = count;
-	  break;
-	}
-      ptr = rs->fields;
+    while(ptr)
+      {
+	server_field = ptr->data;
 
-      field = &row->_buffer[0];
-      colidx = 0;
-      while(ptr)
-	{
-	  server_field = ptr->data;
-	  /* This is commented	  
-	  field->name         = CORBA_string_dup(server_field->name);
-	  field->actualSize   = server_field->actual_length;
-	  field->definedSize  = server_field->defined_length;
-	  field->scale        = server_field->num_scale;
-	  field->precission   = server_field->precission;
-	  field->gdaType      = server_field->value->_d;
-	  field->cType        = server_field->c_type;
-	  field->nativeType   = server_field->sql_type;
-	  */
-	  field->realValue._d = server_field->actual_length == 0;
-	  field->shadowValue._d   = 1;
-	  field->originalValue._d = 1;
-	  //	  if (field->cType == SQL_C_BINARY)
-	  //	    {
-	  //	      fprintf(stderr, "         field->rv._u.v._u.lvb._maximum = %d\n", 
-	  //		      field->realValue._u.v._u.lvb._maximum);
-	  //	      if (field->actualSize == -1)
-	  //		{
-	  //		  field->realValue._u.v._u.lvb._length = 0;
-	  //		  field->realValue._u.v._u.lvb._maximum = 0;
-	  //		  /* hei, it's a NULL value */
-	  //		  CORBA_free(field->realValue._u.v._u.lvb._buffer);
-	  //		  field->realValue._d = 1;
-	  //		}
-	  //	      else
-	  //		field->realValue._u.v._u.lvb._length = field->actualSize;
-	  //	    }
-#if 0
-	  printf("Values for field '%s'\n", field->name);
-	  printf("        field->actualSize = %d\n", field->actualSize);
-	  if (field->cType == SQL_C_BINARY)
-	    {
-	      printf("         field->rv._u.v._u.lvb._length  = %d\n", 
-		     field->realValue._u.v._u.lvb._length);
-	      printf("         field->rv._u.v._u.lvb._maximum = %d\n", 
-		     field->realValue._u.v._u.lvb._maximum);
-	    }
-	  printf("        field->definedSize = %d\n", field->definedSize);
-	  printf("        field->scale       = %d\n", field->scale);
-	  printf("        field->precission  = %d\n", field->precission);
-	  printf("        field->gdaType     = %d\n", field->gdaType);
-	  printf("        field->c_type      = %d\n", field->cType);
-	  
-	  printf("        field->nativeType  = %d\n", field->nativeType);
-	  printf("        field->realVal._d  = %d\n", field->realValue._d);
-	  printf("        field->rv._u.v._d  = %d\n", field->realValue._u.v._d);
-	  printf("        field->rv._u.v._u.ll = %Ld\n", field->realValue._u.v._u.ll);
-#endif
-	  ptr = g_list_next(ptr);
-	  field = &row->_buffer[++colidx];
-	}
-      rowidx++;
+	field->actualSize = server_field->actual_length;
+	field->realValue._d = server_field->actual_length == 0;
+	field->shadowValue._d   = 1;
+	field->originalValue._d = 1;
+	ptr = g_list_next(ptr);
+	field = &row->_buffer[++colidx];
+      }
+    rowidx++;
   }while (rowidx < count);
+
   printf("gda_Recset_fetch: count = %d, rowidx = %d, fetchresult = %d\n", count, rowidx, rc);
+
+  if (rc < 0)
+    {
+      GDA_DriverError*     exception = GDA_DriverError__alloc();
+      Gda_POSTGRES_Connection* cnc = rs->cmd->cnc;
+      
+      fprintf(stderr,"Ooops an error ocurred while fetching data\n");
+      exception->errors._length = g_list_length(cnc->errors);
+      exception->errors._buffer = make_errorbuffer(cnc);
+      exception->realcommand = CORBA_string_dup("Fetch");
+      CORBA_exception_set(ev, CORBA_USER_EXCEPTION, ex_GDA_DriverError, exception);
+      return CORBA_OBJECT_NIL;
+    }
+  chunk = GDA_Recordset_Chunk__alloc();
+  if (rowidx)
+    {
+      chunk->_buffer = CORBA_sequence_GDA_Row_allocbuf(rowidx);
+    }
+  else
+    {
+      chunk->_buffer = 0;
+    }
   chunk->_length = rowidx;
+  for (ptr = tmp_rows, count = 0; count < rowidx; ptr = g_list_next(ptr), count++)
+    {
+      row = ptr->data;
+      chunk->_buffer[count]._length = row->_length;
+      chunk->_buffer[count]._buffer = row->_buffer;
+      g_free(row);
+    }
+  g_list_free(tmp_rows);
   return chunk;
 }
 
@@ -1745,7 +1765,7 @@ impl_GDA_Recordset_describe(impl_POA_GDA_Recordset * servant,
       field->definedSize = server_field->defined_length;
       
       field->scale       = server_field->num_scale;
-      //field->gdaType     = gda_postgres_sql2gdatype(server_field->sql_type);
+      field->gdaType     = GDA_TypeVarchar;
       field->nativeType  = server_field->sql_type;
       ptr = g_list_next(ptr);
       idx++;
