@@ -153,8 +153,6 @@ main(int argc, char* argv[])
   getchar();
   fprintf(stderr,"Continuing\n");
 
-  row = gda_recordset_move(rs, 1, 0);
-
   printf("%s", "Position ");
   for (fieldidx = 0; fieldidx < gda_recordset_rowsize(rs); fieldidx++)
     {
@@ -166,7 +164,7 @@ main(int argc, char* argv[])
   printf("\n");
 
   
-  do 
+  while ((row = gda_recordset_move(rs, 1, 0)) != GDA_RECORDSET_INVALID_POSITION && !gda_recordset_eof(rs))
     {
       printf("%010d", row);
       for (fieldidx = 0; fieldidx < gda_recordset_rowsize(rs); fieldidx++)
@@ -176,7 +174,7 @@ main(int argc, char* argv[])
 	}
       printf("\n");
       row = gda_recordset_move(rs, 1, 0);
-    } while (!gda_recordset_eof(rs));
+    }
 
 
   if (gda_connection_close(cnc) < 0)
