@@ -100,13 +100,9 @@ main(int argc, char* argv[])
       fprintf(stderr,"No recordset from gda_open_schema\n");
       return -1;
     }
-  
-  position = gda_recordset_move(rs, 1, 0);
-  if (position == GDA_RECORDSET_INVALID_POSITION)
-    {
-      fprintf(stderr,"Recordset doesn't contain any data\n");
-      return -1;
-    }
+
+  printf("Press enter to begin fetching data\n");
+  getchar();
   printf("%s", "Position ");
   for (fieldidx = 0; fieldidx < gda_recordset_rowsize(rs); fieldidx++)
     {
@@ -116,8 +112,11 @@ main(int argc, char* argv[])
 	}
     }
   printf("\n");
-  do 
+  while(1)
     {
+      position = gda_recordset_move(rs, 1, 0);
+      if (gda_recordset_eof(rs))
+	  break;
       printf("%010d", row);
       for (fieldidx = 0; fieldidx < gda_recordset_rowsize(rs); fieldidx++)
 	{
@@ -125,9 +124,8 @@ main(int argc, char* argv[])
 	  printf("%15.15s", gda_stringify_value(vbfr,sizeof(vbfr), field));
 	}
       printf("\n");
-      position = gda_recordset_move(rs, 1, 0);
       row++;
-    } while (!gda_recordset_eof(rs));
+    } 
   if (gda_connection_close(cnc) < 0)
     {
       fprintf(stderr,"gda_connection_close failed\n");
